"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright (c) 2015-present, CWB SAS
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * This source code is licensed under the MIT license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * LICENSE file in the root directory of this source tree.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


require("whatwg-fetch");

var _urlBuilder = require("../utils/urlBuilder");

var _urlBuilder2 = _interopRequireDefault(_urlBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var AuthService = function () {
  function AuthService(client) {
    _classCallCheck(this, AuthService);

    if (client && client.provider) {
      throw new Error("Provider authentification not implemented");
    } else if (!client || !client.clientId || !client.clientSecret || !client.url) {
      throw new Error("AuthClient not configured");
    }
    this.client = _extends({}, client);
    this.urlBuilder = new _urlBuilder2.default(client.url);
    this.resetAttributes();
    this.needToLoad = true;
  }

  _createClass(AuthService, [{
    key: "resetAttributes",
    value: function resetAttributes() {
      this.authorized = false;
      this.accessToken = null;
      this.expiresIn = null;
      this.scope = null;
    }
  }, {
    key: "loadAttributes",
    value: function loadAttributes() {
      /* global window */
      this.accessToken = window.localStorage.getItem("access_token");
      this.expiresIn = window.localStorage.getItem("expires_in");
      this.scope = window.localStorage.getItem("scope");
      this.needToLoad = false;
    }
  }, {
    key: "checkAuthentification",
    value: function checkAuthentification() {
      var shouldReset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      if (!this.accessToken) {
        this.loadAttributes();
      } else if (shouldReset) {
        this.resetAttributes();
      }
    }
  }, {
    key: "isAuthenticated",
    value: function isAuthenticated() {
      this.checkAuthentification();
      return this.accessToken != null;
    }
  }, {
    key: "isAccessTokenValid",
    value: function isAccessTokenValid() {
      this.checkAuthentification(true);
      return !!this.accessToken;
    }
  }, {
    key: "getClientId",
    value: function getClientId() {
      return this.client.clientId;
    }
  }, {
    key: "authenticateUser",
    value: async function authenticateUser(_ref) {
      var username = _ref.username,
          password = _ref.password;

      // Authorization request
      // TODO password salt
      // console.log("WIP", `AuthService.requestAccessToken
      // ${username}${password} ${this.client.url}`);
      var url = this.urlBuilder.createUrl("access_token/");

      var body = {
        username: username,
        password: password,
        client_id: this.client.clientId,
        redirect_uri: "localhost",
        grant_type: "password"
      };

      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "POST"
        });
        if (response.status < 200 || response.status > 300) {
          this.resetAccess();
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          this.resetAccess();
          throw session.error;
        }

        this.accessToken = session.access_token;
        window.localStorage.setItem("access_token", this.accessToken);
        this.expiresIn = session.expires_in;
        window.localStorage.setItem("expires_in", this.expiresIn);
        this.scope = session.scope;
        window.localStorage.setItem("scope", this.scope);
        this.authorized = true;
        window.localStorage.setItem("authorized", "true");
        return this.getAttributes();
      } catch (error) {
        this.resetAccess();
        throw error;
      }
    }
  }, {
    key: "authorizeUser",
    value: async function authorizeUser(_ref2) {
      var username = _ref2.username,
          password = _ref2.password,
          scope = _ref2.scope;

      var url = this.urlBuilder.createUrl("authorize/");

      var body = {
        username: username,
        password: password,
        scope: scope,
        client_id: this.client.clientId
      };
      if (this.scope === null) {
        this.resetAccess();
      }

      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "POST"
        });
        if (response.status < 200 || response.status > 300) {
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          throw session.error;
        }

        return session;
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "createUser",
    value: async function createUser(_ref3) {
      var username = _ref3.username,
          email = _ref3.email,
          password = _ref3.password,
          accept = _ref3.accept;

      var url = this.urlBuilder.createUrl("user");
      url = this.buildAuthUrl(url.href);

      var body = {
        username: username,
        email: email,
        password: password,
        accept: accept,
        client_id: this.client.clientId
      };
      if (this.scope === null) {
        this.resetAccess();
      }

      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "POST"
        });
        if (response.status < 200 || response.status > 300) {
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          throw session.error;
        }

        return session;
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "updateAccountState",
    value: async function updateAccountState(_ref4) {
      var userId = _ref4.userId,
          newState = _ref4.newState;

      var url = this.urlBuilder.createUrl("validate");
      url = this.buildAuthUrl(url.href);

      var body = {
        userId: userId,
        newState: newState,
        client_id: this.client.clientId
      };
      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "POST"
        });
        if (response.status < 200 || response.status > 300) {
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          throw session.error;
        }

        return session;
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "logoutUser",
    value: async function logoutUser() {
      var url = this.urlBuilder.createUrl("logout");
      url = this.buildAuthUrl(url.href);

      var body = { client_id: this.client.clientId };

      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "POST"
        });
        this.resetAccess();
        if (response.status < 200 || response.status > 300) {
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          throw session.error;
        }

        return session;
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "lostPassword",
    value: async function lostPassword(_ref5) {
      var email = _ref5.email;

      var url = this.urlBuilder.createUrl("lost_password");
      var body = {
        email: email,
        client_id: this.client.clientId
      };

      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "POST"
        });
        if (response.status < 200 || response.status > 300) {
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          throw session.error;
        }

        return session;
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "resetPassword",
    value: async function resetPassword(_ref6) {
      var newPassword = _ref6.newPassword,
          email = _ref6.email,
          resetToken = _ref6.resetToken;

      var url = this.urlBuilder.createUrl("reset_password");
      url += "?access_token=" + resetToken;
      var body = {
        password: newPassword,
        email: email,
        client_id: this.client.clientId
      };

      try {
        var response = await fetch(url, {
          body: JSON.stringify(body),
          headers: {
            "content-type": "application/json"
          },
          method: "PUT"
        });
        if (response.status < 200 || response.status > 300) {
          throw await AuthService.serializeError(response);
        }

        var session = await response.json();
        if (session.error) {
          throw session.error;
        }

        return session;
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "getAttributes",
    value: function getAttributes() {
      return {
        accessToken: this.accessToken,
        expiresIn: this.expiresIn,
        scope: this.scope
      };
    }
  }, {
    key: "resetAccess",
    value: function resetAccess() {
      this.resetAttributes();
      /* global window */
      window.localStorage.removeItem("authorized");
      window.localStorage.removeItem("access_token");
      window.localStorage.removeItem("expires_in");
      window.localStorage.removeItem("scope");
    }
  }, {
    key: "buildAuthUrl",
    value: function buildAuthUrl(url) {
      if (this.isAuthenticated()) {
        var n = "?access_token=";
        if (url.indexOf("?") > -1) {
          n = "&access_token=";
        }

        return url + n + this.accessToken;
      }
      // TODO check if auth is ok
      return url;
    }
  }], [{
    key: "serializeError",
    value: async function serializeError(response) {
      var res = (await response.json()).error;
      // Simple error response like {"error":"Can't authenticate."}
      var type = "error";
      var message = res;

      // More complexe error like {"error":{"message":"Can't authenticate.","type":"info"}}
      if (res instanceof Object) {
        message = res.message;
        type = res.type;
      }
      var error = {
        message: message,
        type: type,
        status: response.status
      };
      throw error;
    }
  }]);

  return AuthService;
}();

exports.default = AuthService;
//# sourceMappingURL=authService.js.map