"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright (c) 2015-present, CWB SAS
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * This source code is licensed under the MIT license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * LICENSE file in the root directory of this source tree.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


require("whatwg-fetch");

var _urlBuilder = require("../utils/urlBuilder");

var _urlBuilder2 = _interopRequireDefault(_urlBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var WebService = function () {
  function WebService(client, authService) {
    _classCallCheck(this, WebService);

    if (client && client.provider) {
      throw new Error("Provider authentification not implemented");
    } else if (!client || !client.url) {
      throw new Error("WebClient not configured");
    }
    this.urlBuilder = new _urlBuilder2.default(client.url);
    this.client = _extends({}, client);
    this.authService = authService;
  }

  _createClass(WebService, [{
    key: "buildHttpUrl",
    value: function buildHttpUrl(route) {
      var url = this.urlBuilder.createUrl(route);
      return this.authService.buildAuthUrl(url.href);
    }
  }, {
    key: "buildWsUrl",
    value: function buildWsUrl(route) {
      var url = this.urlBuilder.createWsUrl(route);
      return this.authService.buildAuthUrl(url.href);
    }
  }, {
    key: "send",
    value: async function send(route, data, method, auth) {
      var isAuth = this.authService.isAuthenticated();
      var clientId = this.authService.getClientId();
      var url = this.buildHttpUrl(route);

      if (!isAuth && auth) {
        throw new Error("not authenticated");
      }

      var headers = {
        client_id: clientId,
        Accept: "application/json"
      };

      var config = {
        method: method
      };

      if (data !== null) {
        headers["Content-Type"] = "application/json";
        config.body = JSON.stringify(data);
      }

      config.headers = headers;

      try {
        var response = await fetch(url, config);
        if (response.status < 200 || response.status > 300) {
          var res = (await response.json()).error;
          // Simple error response like {"error":"Can't authenticate."}
          var type = "error";
          var message = res;

          // More complexe error like {"error":{"message":"Can't authenticate.","type":"info"}}
          if (res instanceof Object) {
            message = res.message;
            type = res.type;
          }
          var error = {
            message: message,
            type: type,
            status: response.status
          };
          throw error;
        }

        return response.json();
      } catch (error) {
        throw error;
      }
    }
  }, {
    key: "get",
    value: function get(route) {
      var auth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      return this.send(route, null, "get", auth);
    }
  }, {
    key: "post",
    value: function post(route, inputData) {
      var auth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

      return this.send(route, inputData, "post", auth);
    }
  }, {
    key: "put",
    value: function put(route, inputData) {
      var auth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

      return this.send(route, inputData, "put", auth);
    }
  }, {
    key: "delete",
    value: function _delete(route) {
      var auth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      return this.send(route, null, "delete", auth);
    }
  }]);

  return WebService;
}();

exports.default = WebService;
//# sourceMappingURL=webService.js.map